from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

# Reference: https://cloud.google.com/iam/docs/best-practices-service-accounts
# Lookup: https://cloud.google.com/iam/docs/permissions-reference

IMPERSONATION_ROLES = [
    "roles/owner",
    "roles/editor",
    "roles/iam.securityAdmin",
    "roles/iam.serviceAccountAdmin",
    "roles/iam.serviceAccountKeyAdmin",
    "roles/iam.serviceAccountUser",
    "roles/iam.serviceAccountTokenCreator",
    "roles/iam.workloadIdentityUser",
    "roles/dataflow.developer",
    "roles/serverless.serviceAgent",
    "roles/dataproc.serviceAgent",
    "roles/deploymentmanager.editor",
    "roles/cloudbuild.builds.editor",
    "roles/aiplatform.customCodeServiceAgent",
    "roles/aiplatform.extensionServiceAgent",
    "roles/aiplatform.serviceAgent",
    "roles/apigateway.serviceAgent",
    "roles/apigee.serviceAgent",
    "roles/appengine.serviceAgent",
    "roles/appengineflex.serviceAgent",
    "roles/bigquerycontinuousquery.serviceAgent",
    "roles/bigquerydatatransfer.serviceAgent",
    "roles/bigqueryspark.serviceAgent",
    "roles/cloudbuild.serviceAgent",
    "roles/cloudconfig.serviceAgent",
    "roles/clouddeploy.serviceAgent",
    "roles/cloudfunctions.serviceAgent",
    "roles/cloudscheduler.serviceAgent",
    "roles/cloudtasks.serviceAgent",
    "roles/composer.serviceAgent",
    "roles/compute.serviceAgent",
    "roles/connectors.serviceAgent",
    "roles/dataflow.serviceAgent",
    "roles/eventarc.serviceAgent",
    "roles/integrations.serviceAgent",
    "roles/ml.serviceAgent",
    "roles/notebooks.serviceAgent",
    "roles/pubsub.serviceAgent",
    "roles/run.serviceAgent",
    "roles/sourcerepo.serviceAgent",
    "roles/workflows.serviceAgent",
    "roles/iam.serviceAccountOpenIdTokenCreator",
    "roles/aiplatform.colabServiceAgent",
    "roles/backupdr.computeEngineOperator",
    "roles/backupdr.serviceAgent",
    "roles/batch.serviceAgent",
    "roles/clouddeploymentmanager.serviceAgent",
    "roles/cloudtpu.serviceAgent",
    "roles/compute.instanceGroupManagerServiceAgent",
    "roles/configdelivery.serviceAgent",
    "roles/container.serviceAgent",
    "roles/datapipelines.serviceAgent",
    "roles/dataplex.serviceAgent",
    "roles/dataprep.serviceAgent",
    "roles/dataproc.hubAgent",
    "roles/firebaseapphosting.serviceAgent",
    "roles/firebasemods.serviceAgent",
    "roles/gameservices.serviceAgent",
    "roles/genomics.serviceAgent",
    "roles/krmapihosting.anthosApiEndpointServiceAgent",
    "roles/krmapihosting.serviceAgent",
    "roles/lifesciences.serviceAgent",
    "roles/osconfig.serviceAgent",
    "roles/runapps.serviceAgent",
    "roles/securitycenter.securityResponseServiceAgent",
    "roles/workstations.serviceAgent",
    "roles/securesourcemanager.serviceAgent",
    "roles/assuredoss.admin",
    "roles/securitycenter.admin",
    "roles/vpcaccess.serviceAgent",
    "roles/cloudbuild.builds.builder",
    "roles/composer.worker",
    "roles/dataflow.admin",
    "roles/run.sourceDeveloper",
]


class AbsGoogleImpersonationRoles(BaseResourceCheck):
    def scan_resource_conf(self, conf):
        self.evaluated_keys = ['role']
        if 'role' in conf and conf['role'][0] in IMPERSONATION_ROLES:
            return CheckResult.FAILED
        return CheckResult.PASSED



from __future__ import annotations

import asyncio
import logging
from abc import abstractmethod
from collections.abc import Iterable
from pathlib import Path
from typing import Any, TYPE_CHECKING, Generic, TypeVar

import aiohttp
import docker

from checkov.common.bridgecrew.vulnerability_scanning.image_scanner import image_scanner
from checkov.common.bridgecrew.vulnerability_scanning.integrations.docker_image_scanning import \
    docker_image_scanning_integration
from checkov.common.output.common import ImageDetails
from checkov.common.output.report import Report, CheckType
from checkov.common.sca.commons import should_run_scan
from checkov.common.sca.output import add_to_report_sca_data, get_license_statuses_async
from checkov.common.typing import _LicenseStatus

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
    from checkov.runner_filter import RunnerFilter
    from networkx import DiGraph

_Definitions = TypeVar("_Definitions")

INVALID_IMAGE_NAME_CHARS = ("[", "{", "(", "<", "$")


def fix_related_resource_ids(report: Report | None, tmp_dir: str) -> None:
    """Remove tmp dir prefix from 'relatedResourceId'"""

    if report and report.image_cached_results:
        for cached_result in report.image_cached_results:
            related_resource_id = cached_result.get("relatedResourceId")
            if related_resource_id and isinstance(related_resource_id, str):
                cached_result["relatedResourceId"] = related_resource_id.replace(tmp_dir, "", 1)


class Image:
    def __init__(self, file_path: str, name: str, start_line: int, end_line: int,
                 related_resource_id: str | None = None) -> None:
        """

        :param file_path: example: 'checkov/integration_tests/example_workflow_file/.github/workflows/vulnerable_container.yaml'
        :param name: example: 'node:14.16'
        :param image_id: example: 'sha256:6a353e22ce'
        :param start_line: example: 8
        :param end_line: example: 16
        """
        self.end_line = end_line
        self.start_line = start_line
        self.name = name
        self.file_path = file_path
        self.related_resource_id = related_resource_id

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__

        return False

    def __ne__(self, other: Any) -> bool:
        return not self.__eq__(other)

    def __hash__(self) -> int:
        return hash((self.file_path, self.name, self.start_line, self.end_line))


class ImageReferencer:
    @abstractmethod
    def is_workflow_file(self, file_path: str) -> bool:
        """

        :param file_path: path of file to validate if it is a file that contains might images (example: CI workflow file)
        :return: True if contains images

        """
        return False

    @abstractmethod
    def get_images(self, file_path: str) -> Iterable[Image]:
        """
        Get container images mentioned in a file
        :param file_path: File to be inspected
        :return: List of container images objects mentioned in the file.
        """
        return []

    @staticmethod
    def inspect(image_name: str) -> str:
        """

        :param image_name: name of the image to be inspected locally using a "docker inspect X". If image does not exist try to pull it locally.
        :return: short image id sha that is inspected. In case inspect has failed None will be returned.
        """
        try:
            logging.info("Inspecting docker image {}".format(image_name))
            client = docker.from_env()
            try:
                image = client.images.get(image_name)
            except Exception:
                image = client.images.pull(image_name)
                return image.short_id
            return image.short_id
        except Exception:
            logging.debug(f"failed to pull docker image={image_name}", exc_info=True)
            return ""


def is_valid_public_image_name(image_name: str) -> bool:
    if image_name.startswith('localhost'):
        return False
    if any(char in image_name for char in INVALID_IMAGE_NAME_CHARS):
        return False
    if image_name.count(":") > 1:
        # if there is more than one colon, then it is typically a private registry with port reference
        return False
    return True


class ImageReferencerMixin(Generic[_Definitions]):
    """Mixin class to simplify image reference search"""

    def check_container_image_references(
        self,
        root_path: str | Path | None,
        runner_filter: RunnerFilter,
        graph_connector: DiGraph | None = None,
        definitions: _Definitions | None = None,
        definitions_raw: dict[str, list[tuple[int, str]]] | None = None,
    ) -> Report | None:
        """Tries to find image references in graph based IaC templates"""
        from checkov.common.bridgecrew.platform_integration import bc_integration

        # skip complete run, if flag '--check' was used without a CVE check ID
        if not should_run_scan(runner_filter.checks):
            return None

        images = self.extract_images(graph_connector=graph_connector, definitions=definitions,
                                     definitions_raw=definitions_raw)
        if not images:
            return None

        logging.info(f"Found {len(images)} image references {[image.name for image in images]}")

        report = Report(CheckType.SCA_IMAGE)
        root_path = Path(root_path) if root_path else None
        check_class = f"{image_scanner.__module__}.{image_scanner.__class__.__qualname__}"
        report_type = CheckType.SCA_IMAGE
        image_names_to_query = list(set(filter(lambda i: is_valid_public_image_name(i), map(lambda i: i.name, images))))
        results = asyncio.run(self._fetch_image_results_async(image_names_to_query))

        license_statuses_by_image = asyncio.run(self._fetch_licenses_per_image(image_names_to_query, results))

        for image in images:
            try:
                results_index = image_names_to_query.index(image.name)
                cached_results = results[results_index]
            except ValueError:
                cached_results = {}

            file_line_range = [image.start_line, image.end_line]
            self._add_image_records(
                report=report,
                root_path=root_path,
                check_class=check_class,
                dockerfile_path=image.file_path,
                image=image,
                runner_filter=runner_filter,
                report_type=report_type,
                bc_integration=bc_integration,
                cached_results=cached_results,
                license_statuses=license_statuses_by_image.get(image.name) or [],
                file_line_range=file_line_range if None not in file_line_range else None
            )

        return report

    @staticmethod
    async def _fetch_image_results_async(image_names_to_query: list[str]) -> list[dict[str, Any]]:
        """
        This is an async implementation of `_fetch_image_results`. The only change is we're getting a session
        as an input, and the asyncio behavior is managed in the calling method.
        """
        async with aiohttp.ClientSession() as session:
            results: list[dict[str, Any]] = await asyncio.gather(*[
                image_scanner.get_scan_results_from_cache_async(session, f"image:{i}")
                for i in image_names_to_query
            ])
        return results

    def _add_image_records(
        self,
        report: Report,
        root_path: Path | None,
        check_class: str,
        dockerfile_path: str,
        image: Image,
        runner_filter: RunnerFilter,
        report_type: str,
        bc_integration: BcPlatformIntegration,
        cached_results: dict[str, Any],
        license_statuses: list[_LicenseStatus],
        file_line_range: list[int] | None = None
    ) -> None:
        """Adds an image record to the given report, if possible"""
        if cached_results:
            logging.info(f"Found cached scan results of image {image.name}")
            image_scanning_report: dict[str, Any] = docker_image_scanning_integration.create_report(
                twistcli_scan_result=cached_results,
                bc_platform_integration=bc_integration,
                file_path=dockerfile_path,
                file_content=f'image: {image.name}',
                docker_image_name=image.name,
                related_resource_id=image.related_resource_id,
                root_folder=root_path,
                error_lines=file_line_range
            )
            report.image_cached_results.append(image_scanning_report)

            result = cached_results.get("results", [{}])[0]
            image_id = self._extract_image_short_id(result)
            image_details = self._get_image_details_from_twistcli_result(scan_result=result, image_id=image_id)
            dockerfile_rel_path = dockerfile_path
            if root_path:
                try:
                    dockerfile_rel_path = str(Path(dockerfile_path).relative_to(root_path))
                except ValueError:
                    # Path.is_relative_to() was implemented in Python 3.9
                    pass
            rootless_file_path = dockerfile_rel_path.replace(Path(dockerfile_rel_path).anchor, "", 1)
            rootless_file_path_to_report = f"{rootless_file_path} ({image.name} lines:{image.start_line}-" \
                                           f"{image.end_line} ({image_id}))"

            self._add_vulnerability_records(
                report=report,
                result=result,
                check_class=check_class,
                dockerfile_path=dockerfile_path,
                rootless_file_path=rootless_file_path_to_report,
                image_details=image_details,
                runner_filter=runner_filter,
                report_type=report_type,
                license_statuses=license_statuses,
                file_line_range=file_line_range
            )
        else:
            logging.info(f"No cache hit for image {image.name}")

    @staticmethod
    def _extract_image_short_id(scan_result: dict[str, Any]) -> str:
        """Extracts a shortened version of the image ID from the scan result"""

        if "id" not in scan_result:
            return "sha256:unknown"

        image_id: str = scan_result["id"]

        if image_id.startswith("sha256:"):
            return image_id[:17]
        return image_id[:10]

    @staticmethod
    def _get_image_details_from_twistcli_result(scan_result: dict[str, Any], image_id: str) -> ImageDetails:
        """Extracts the image detaisl from a twistcli scan result"""

        image_packages = scan_result.get("packages", [])
        image_package_types = {f'{package["name"]}@{package["version"]}': package["type"] for package in image_packages}
        return ImageDetails(
            distro=scan_result.get("distro", ""),
            distro_release=scan_result.get("distroRelease", ""),
            package_types=image_package_types,
            image_id=image_id,
        )

    @staticmethod
    def _add_vulnerability_records(
        report: Report,
        result: dict[str, Any],
        check_class: str,
        dockerfile_path: str,
        rootless_file_path: str,
        image_details: ImageDetails | None,
        license_statuses: list[_LicenseStatus],
        runner_filter: RunnerFilter,
        report_type: str,
        file_line_range: list[int] | None = None
    ) -> None:
        vulnerabilities = result.get("vulnerabilities", [])
        packages = result.get("packages", [])
        add_to_report_sca_data(
            report=report,
            check_class=check_class,
            scanned_file_path=dockerfile_path,
            rootless_file_path=rootless_file_path,
            runner_filter=runner_filter,
            vulnerabilities=vulnerabilities,
            packages=packages,
            license_statuses=license_statuses,
            sca_details=image_details,
            report_type=report_type,
            file_line_range=file_line_range
        )

    @abstractmethod
    def extract_images(
        self,
        graph_connector: DiGraph | None = None,
        definitions: _Definitions | None = None,
        definitions_raw: dict[str, list[tuple[int, str]]] | None = None
    ) -> list[Image]:
        """Tries to find image references in the graph or supported resource"""

        pass

    @staticmethod
    async def _fetch_licenses_per_image(image_names: list[str], image_results: list[dict[str, Any]]) \
            -> dict[str, list[_LicenseStatus]]:
        merged_result: dict[str, list[_LicenseStatus]] = {}
        async with aiohttp.ClientSession() as session:
            license_results = await asyncio.gather(*[
                get_license_statuses_async(session, result['results'][0].get('packages') or [], image_names[i])
                for i, result in enumerate(image_results)
                if "results" in result and result["results"]
            ])
        merged_result.update({r['image_name']: r['licenses'] for r in license_results})
        return merged_result



from __future__ import annotations

import json
import logging
from collections.abc import Iterable
from itertools import groupby
from typing import TYPE_CHECKING, Any

from urllib3 import PoolManager
from urllib3.exceptions import ProtocolError

from checkov.common.bridgecrew.integration_features.base_integration_feature import BaseIntegrationFeature
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.util.data_structures_utils import merge_dicts
from checkov.common.util.http_utils import extract_error_message, get_default_post_headers

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
    from checkov.common.output.record import Record
    from checkov.common.output.report import Report
    from checkov.common.typing import _BaseRunner

SUPPORTED_FIX_FRAMEWORKS = ['terraform', 'cloudformation']


class FixesIntegration(BaseIntegrationFeature):
    def __init__(self, bc_integration: BcPlatformIntegration) -> None:
        super().__init__(bc_integration=bc_integration, order=10)

    @property
    def fixes_url(self) -> str:
        return f"{self.bc_integration.api_url}/api/v1/fixes/checkov"

    def is_valid(self) -> bool:
        return (
            self.bc_integration.is_integration_configured()
            and not self.bc_integration.skip_fixes
            and not self.bc_integration.on_prem
            and not self.bc_integration.skip_download
            and not self.integration_feature_failures
        )

    def post_runner(self, scan_report: Report) -> None:
        try:
            if scan_report.check_type not in SUPPORTED_FIX_FRAMEWORKS:
                return
            self._get_platform_fixes(scan_report)
        except Exception:
            self.integration_feature_failures = True
            logging.debug("Fixes will not be applied.", exc_info=True)

    def _get_platform_fixes(self, scan_report: Report) -> None:

        # We might want to convert this to one call for all results (all files), but then we would also have to deal
        # with repo size issues. Because the primary use case for this at the moment is VSCode integration, which
        # runs one file at a time, this can wait.

        sorted_by_file = sorted(scan_report.failed_checks, key=lambda c: c.file_abs_path)
        for file, sorted_failed_checks in groupby(sorted_by_file, key=lambda c: c.file_abs_path):
            failed_checks = [fc for fc in sorted_failed_checks if fc.check_id in metadata_integration.check_metadata]
            if not failed_checks:
                continue
            with open(file, 'r') as reader:
                file_contents = reader.read()

            fixes = self._get_fixes_for_file(scan_report.check_type, file, file_contents, failed_checks)
            if not fixes:
                continue
            all_fixes = fixes['fixes']

            # a mapping of (checkov_check_id, resource_id) to the failed check Record object for lookup later
            # guaranteed to map to exactly one record
            failed_check_by_check_resource: dict[tuple[str, str], Record] = {
                k: list(v)[0] for k, v in groupby(failed_checks, key=lambda c: (c.check_id, c.resource))
            }

            for fix in all_fixes:
                ckv_id = metadata_integration.get_ckv_id_from_bc_id(fix['policyId'])
                if not ckv_id:
                    logging.debug(f"BC ID {fix['policyId']} has no checkov ID - might be a cloned policy")
                    ckv_id = fix.get('policyId', '')

                failed_check = failed_check_by_check_resource.get((ckv_id, fix['resourceId']))  # type:ignore[arg-type]  # ckv_id is not None here
                if not failed_check:
                    logging.warning(f'Could not find the corresponding failed check for the fix for ID {ckv_id} and resource {fix["resourceId"]}')
                    continue
                failed_check.fixed_definition = fix['fixedDefinition']

    def _get_fixes_for_file(
        self, check_type: str, filename: str, file_contents: str, failed_checks: Iterable[Record]
    ) -> dict[str, Any] | None:
        if not self.bc_integration.bc_source:
            logging.error("Source was not set")
            return None

        errors = list(map(lambda c: {
            'resourceId': c.resource,
            'policyId': metadata_integration.get_bc_id(c.check_id) or c.check_id,
            'startLine': c.file_line_range[0],
            'endLine': c.file_line_range[1]
        }, failed_checks))

        payload = {
            'filePath': filename,
            'fileContent': file_contents,
            'framework': check_type,
            'errors': errors
        }
        logging.debug(f'Payload for fixes API: file_path: {filename}, fileContent: {file_contents}, framework: {check_type}, errors: {errors}')

        headers = merge_dicts(
            get_default_post_headers(self.bc_integration.bc_source, self.bc_integration.bc_source_version),
            {"Authorization": self.bc_integration.get_auth_token()},
            self.bc_integration.custom_auth_headers
        )

        if not self.bc_integration.http:
            raise AttributeError("HTTP manager was not correctly created")

        try:
            logging.debug(f'Calling fixes API with payload: {json.dumps(payload)}, headers: {headers}, url: {self.fixes_url}')
            request = self.bc_integration.http.request("POST", self.fixes_url, headers=headers, body=json.dumps(payload))  # type:ignore[no-untyped-call]

        # When running via IDE we can fail here in case of running with -d when the poolManager is broken
        except ProtocolError as e:
            logging.error(f'Get fixes request for file {filename} failed with response code error: {e}')
            if isinstance(self.bc_integration.http, PoolManager):
                self.bc_integration.http = None
                self.bc_integration.setup_http_manager(
                    self.bc_integration.ca_certificate,
                    self.bc_integration.no_cert_verify
                )
                request = self.bc_integration.http.request("POST", self.fixes_url, headers=headers, body=json.dumps(payload))  # type:ignore
            else:
                return None

        if request.status != 200:
            error_message = extract_error_message(request)
            logging.error(f'Get fixes request for file {filename} failed with response code {request.status}: {error_message} - skipping fixes for this file')
            return None

        logging.debug(f'Response from fixes API: {request.data}')

        fixes: list[dict[str, Any]] | None = json.loads(request.data) if request.data else None
        if not fixes or not isinstance(fixes, list):
            logging.warning(f'Unexpected fixes API response for file {filename}; skipping fixes for this file')
            return None
        return fixes[0]

    def pre_scan(self) -> None:
        # not used
        pass

    def pre_runner(self, runner: _BaseRunner) -> None:
        # not used
        pass

    def post_scan(self, merged_reports: list[Report]) -> None:
        # not used
        pass


integration = FixesIntegration(bc_integration)



from __future__ import annotations
from typing import Any
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck


class AzureDefenderOnKubernetes(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Defender is set to On for Kubernetes"
        id = "CKV_AZURE_85"
        supported_resources = ("Microsoft.Security/pricings",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        return (
            CheckResult.PASSED
            if conf.get("name") != "KubernetesService" or str(conf["properties"]["pricingTier"]).lower() == "standard"
            else CheckResult.FAILED
        )

    def get_evaluated_keys(self) -> list[str]:
        return ["name", "pricingTier"]


check = AzureDefenderOnKubernetes()



from __future__ import annotations

from typing import Any, Dict

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck


class AzureDefenderOnKeyVaults(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Defender is set to On for Key Vault"
        id = "CKV_AZURE_87"
        supported_resources = ("Microsoft.Security/pricings",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        properties = conf.get('properties', {})
        pricing_tier = properties.get('pricingTier')
        name = conf.get('name', '')
        return (
            CheckResult.PASSED
            if pricing_tier == "Standard" and name == 'KeyVaults'
            else CheckResult.FAILED
        )

    def get_evaluated_keys(self) -> list[str]:
        return ["properties.pricingTier", "name"]


check = AzureDefenderOnKeyVaults()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck
from typing import List, Dict, Any


class AzureDefenderOnSqlServersVMS(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Defender is set to On for SQL servers on machines"
        id = "CKV_AZURE_79"
        supported_resources = ("Microsoft.Security/pricings",)
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        properties = conf.get("properties", {})
        name = conf.get("name", "")
        tier = properties.get("tier")
        if tier == "Standard" and name == "SqlServerVirtualMachines":
            return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ["tier"]


check = AzureDefenderOnSqlServersVMS()



from __future__ import annotations

from typing import Dict

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class AppServiceInstanceMinimum(BaseResourceCheck):
    def __init__(self) -> None:
        # "App Services Plans provides a configurable number of instances that will run apps.
        # When a single instance is configured your app may be temporarily unavailable during unplanned interruptions.
        # In most circumstances, Azure will self-heal faulty app service instances automatically.
        # How-ever during this time there may interruptions to your workload."
        name = "Ensure App Service has a minimum number of instances for failover"
        id = "CKV_AZURE_212"
        supported_resources = ("Microsoft.Web/sites", "Microsoft.Web/sites/slots")
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Dict[str, Dict[str, int]]]) -> CheckResult:
        if "properties" in conf:
            if conf.get("properties", {}).get("siteConfig") is not None:
                if "numberOfWorkers" in conf["properties"]["siteConfig"]:
                    worker_count = conf["properties"]["siteConfig"]["numberOfWorkers"]
                    if worker_count:
                        if not isinstance(worker_count, int):
                            return CheckResult.UNKNOWN
                        if worker_count > 1:
                            return CheckResult.PASSED
        return CheckResult.FAILED


check = AppServiceInstanceMinimum()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SynapseSQLPoolDataEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Synapse SQL pools are encrypted"
        id = "CKV_AZURE_241"
        supported_resources = ['azurerm_synapse_sql_pool']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'data_encrypted' in conf and conf['data_encrypted'][0] is True:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = SynapseSQLPoolDataEncryption()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck

LOCATIONS_W_REDUNDANCY = [
    "Brazil South", "France Central", "Qatar Central", "South Africa North", "Australia East",
    "Canada Central", "Italy North", "UAE North", "Central India",
    "Central US", "Germany West Central", "Israel Central", "Japan East",
    "East US", "Norway East", "Japan West",
    "East US 2", "North Europe", "Southeast Asia",
    "South Central US", "UK South", "East Asia",
    "US Gov Virginia", "West Europe", "China North 3",
    "West US 2", "Sweden Central", "Korea Central",
    "West US 3", "Switzerland North", "New Zealand North",
    "Mexico Central", "Poland Central",
    "Spain Central"
]


class EventHubNamespaceZoneRedundant(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        Zone Redundancy is now determined automatically based on region.
        """
        name = "Ensure the Azure Event Hub Namespace is zone redundant"
        id = "CKV_AZURE_228"
        supported_resources = ("azurerm_eventhub_namespace",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "location"

    def get_expected_values(self):
        return LOCATIONS_W_REDUNDANCY


check = EventHubNamespaceZoneRedundant()



from typing import Any, List

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class Route53TransferLock(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure Route 53 domains have transfer lock protection"
        id = "CKV_AWS_377"
        supported_resources = ('aws_route53domains_registered_domain',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'transfer_lock'

    def get_forbidden_values(self) -> List[Any]:
        return [False]


check = Route53TransferLock()



import re
from typing import Any, Dict

from checkov.common.models.consts import DOCKER_IMAGE_REGEX
from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class ImagePullPolicyAlways(BaseK8sContainerCheck):
    def __init__(self) -> None:
        """
        Image pull policy should be set to always to ensure you get the correct image and imagePullSecrets are correct
        Default is 'IfNotPresent' unless image tag is omitted or :latest
        https://kubernetes.io/docs/concepts/configuration/overview/#container-images

        An admission controller could be used to enforce imagePullPolicy
        """
        name = "Image Pull Policy should be Always"
        id = "CKV_K8S_15"
        # Location: container .imagePullPolicy
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["image", "imagePullPolicy"]
        if conf.get("image"):
            # Remove the digest, if present
            image_val = conf["image"]
            if not isinstance(image_val, str) or image_val.strip() == "":
                return CheckResult.UNKNOWN
            if "@" in image_val:
                image_val = image_val[0 : image_val.index("@")]
                has_digest = True
            else:
                has_digest = False
            if "imagePullPolicy" not in conf:
                image_tag_match = re.findall(DOCKER_IMAGE_REGEX, image_val)
                if len(image_tag_match) != 1:
                    # If image name is invalid, check result can not be determined
                    return CheckResult.UNKNOWN
                (image, tag) = image_tag_match[0]
                if tag == "latest" or tag == "":
                    # Default imagePullPolicy = Always
                    return CheckResult.PASSED
                elif has_digest:
                    return CheckResult.PASSED
                else:
                    # Default imagePullPolicy = IfNotPresent
                    return CheckResult.FAILED
            else:
                if not has_digest and conf["imagePullPolicy"] != "Always":
                    return CheckResult.FAILED

        else:
            return CheckResult.FAILED
        return CheckResult.PASSED


check = ImagePullPolicyAlways()



from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class EventgridTopicNetworkAccess(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure public network access is disabled for Azure Event Grid Topic"
        id = "CKV_AZURE_193"
        supported_resources = ("Microsoft.EventGrid/topics",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/publicNetworkAccess"

    def get_expected_value(self) -> str:
        return "Disabled"


check = EventgridTopicNetworkAccess()



from __future__ import annotations

from abc import abstractmethod
from typing import Tuple, List, Dict, Any, TYPE_CHECKING

from checkov.common.graph.checks_infra.enums import SolverType
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.util.env_vars_config import env_vars_config

if TYPE_CHECKING:
    from networkx import DiGraph

# Based on the resource names in iac frameworks
AWS_KEYS = ['aws_', 'AWS::', 'aws-']
GCP_KEYS = ['gcloud', 'google_']
AZURE_KEYS = ['azurerm_', 'Microsoft.']


class BaseSolver:
    operator = ""  # noqa: CCE003  # a static attribute

    def __init__(self, solver_type: SolverType) -> None:
        self.solver_type = solver_type
        self.providers: List[str] = []

    @abstractmethod
    def get_operation(self, *args: Any, **kwargs: Any) -> Any:
        raise NotImplementedError()

    @abstractmethod
    def _get_operation(self, *args: Any, **kwargs: Any) -> Any:
        raise NotImplementedError()

    @abstractmethod
    def run(self, graph_connector: DiGraph) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        raise NotImplementedError()

    def resource_type_pred(self, v: Dict[str, Any], resource_types: List[str]) -> bool:
        resource_type = CustomAttributes.RESOURCE_TYPE
        if env_vars_config.CKV_SUPPORT_ALL_RESOURCE_TYPE:
            is_all_resources = isinstance(resource_types, list) and resource_types[0].lower() == "all"
            resource_type_match_provider = self.resource_match_provider(v.get(resource_type, ''))
            support_all_resources = bool(resource_type in v and is_all_resources and v.get(resource_type) != 'module' and resource_type_match_provider)

            return not resource_types or support_all_resources

        return not resource_types or (resource_type in v and v[resource_type] in resource_types)

    def resource_match_provider(self, resource_type: str) -> bool:
        if not self.providers:
            return True
        for provider in self.providers:
            if provider.lower() == 'aws':
                if any(resource_type.startswith(key) for key in AWS_KEYS):
                    return True
            elif provider.lower() == 'gcp':
                if any(resource_type.startswith(key) for key in GCP_KEYS):
                    return True
            elif provider.lower() == 'azure':
                if any(resource_type.startswith(key) for key in AZURE_KEYS):
                    return True
            else:  # if we don't have a provider or the provider was not one of the basic providers
                return True
        return False



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class CloudFrontGeoRestrictionDisabled(BaseResourceNegativeValueCheck):

    def __init__(self) -> None:
        name = "Ensure AWS CloudFront web distribution has geo restriction enabled"
        id = "CKV_AWS_374"
        supported_resources = ('aws_cloudfront_distribution',)
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "restrictions/[0]/geo_restriction/[0]/restriction_type"

    def get_forbidden_values(self) -> list:
        return ["none"]


check = CloudFrontGeoRestrictionDisabled()
